@@ -1,32 +1,40 @@
-#! /bin/sh
+#!/bin/sh
 #
-#	A very simple function test for gdiffmk.sh.
+# Copyright 2004-2023 Free Software Foundation, Inc.
+#
+# Written by Mike Bianchi <MBianchi@Foveal.com>
+# Changes from May 2015 onward by the groff development team
+#
+# This file is part of gdiffmk.
+#
+# gdiffmk is distributed with groff, the GNU roff typesetting system.
 #
-# Copyright (C) 2004-2020, 2023 Free Software Foundation, Inc.
-# Written by Mike Bianchi <MBianchi@Foveal.com>.
-# Subsequent modifications by G. Branden Robinson.
-
-# This file is part of the gdiffmk utility, which is part of groff.
-
 # groff is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-
+#
 # groff is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 # or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 # License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-# This file is part of GNU gdiffmk.
+
+# A very simple function test for gdiffmk.sh.
 
 # abs_top_out_dir is set by AM_TESTS_ENVIRONMENT (defined in
 # Makefile.am) when running "make check".
 
 gdiffmk=${abs_top_out_dir:-.}/gdiffmk
 
+if ! diff -Dx /dev/null /dev/null >/dev/null 2>&1
+then
+    echo "$0: diff command does not support -D option; skipping" >&2
+    exit 77 # skip
+fi
+
 # Locate directory containing our test artifacts.
 in_dir=
 
@@ -49,7 +57,7 @@ done
 # If we can't find it, we can't test.
 if [ -z "$in_dir" ]
 then
-    echo "$0: cannot locate test artifact input directory" >&2
+    echo "$0: cannot locate test artifact input directory; skipping" >&2
     exit 77 # skip
 fi
 
@@ -69,7 +77,7 @@ done
 # If we can't find it, we can't test.
 if [ -z "$out_dir" ]
 then
-    echo "$0: cannot locate test artifact output directory" >&2
+    echo "$0: cannot locate test artifact output directory; skipping" >&2
     exit 77 # skip
 fi
 
@@ -184,4 +192,4 @@ CleanUp
 
 exit ${exit_code}
 
-# vim:set ai et sw=4 ts=4 tw=72:
+# vim:set autoindent expandtab shiftwidth=4 tabstop=4 textwidth=72:
