@@ -1,8 +1,8 @@
 #!/bin/sh
 #
-# Copyright (C) 2022 Free Software Foundation, Inc.
+# Copyright 2022 G. Branden Robinson
 #
-# This file is part of groff.
+# This file is part of groff, the GNU roff typesetting system.
 #
 # groff is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -23,7 +23,7 @@ groff="${abs_top_builddir:-.}/test-groff"
 fail=
 
 wail () {
-    echo FAILED >&2
+    echo ...FAILED >&2
     fail=YES
 }
 
@@ -47,6 +47,7 @@ x = y
 .DE'
 
 output=$(printf "%s\n" "$input" | "$groff" -e -mm -Tascii -P-cbou)
+echo "$output"
 
 echo "checking left-aligned displayed equation" >&2
 echo "$output" | grep -Eq 'p=q {54}\(1\)' || wail
@@ -57,6 +58,6 @@ echo "$output" | grep -Eq 'w=z {49}\(2\)' || wail
 echo "checking centered displayed equation" >&2
 echo "$output" | grep -Eq 'x=y {26}\(3\)' || wail
 
-test -z "$fail" || exit 1
+test -z "$fail"
 
 # vim:set ai et sw=4 ts=4 tw=72:
