@@ -1,7 +1,7 @@
-/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
+/* Copyright 1989-2025 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -19,19 +19,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 #ifndef GROFF_LIB_H
 #define GROFF_LIB_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #if defined(__INTERIX) && !defined(_ALL_SOURCE)
 #define _ALL_SOURCE
 #endif
 
 #ifdef __cplusplus
 extern "C" {
-#endif
-#ifndef HAVE_STRERROR
-  char *strerror(int);
 #endif
   const char *i_to_a(int);
   const char *ui_to_a(unsigned int);
@@ -40,9 +33,6 @@ extern "C" {
 }
 #endif
 
-#define __GETOPT_PREFIX groff_
-#include <getopt.h>
-
 #ifdef HAVE_SETLOCALE
 #include <locale.h>
 #define getlocale(category) setlocale(category, NULL)
@@ -56,8 +46,7 @@ extern "C" {
 #include <stdbool.h>
 
 char *strsave(const char *s);
-bool is_prime(unsigned);
-double groff_hypot(double, double);
+unsigned ceil_prime(unsigned);
 
 #include <stdio.h>
 #include <string.h>
@@ -85,10 +74,9 @@ int mkstemp(char *tmpl);
 int mksdir(char *tmpl);
 
 #ifdef __cplusplus
-  FILE *xtmpfile(char **namep = 0,
-		 const char *postfix_long = 0,
-		 const char *postfix_short = 0,
-		 int do_unlink = 1);
+  FILE *xtmpfile(const char **namep = 0 /* nullptr */,
+		 const char *postfix_long = 0 /* nullptr */,
+		 const char *postfix_short = 0 /* nullptr */);
   char *xtmptemplate(const char *postfix_long,
 		     const char *postfix_short);
 #endif
@@ -111,26 +99,6 @@ int mksdir(char *tmpl);
     return (c >= 0 && invalid_char_table[c]);
   }
 
-#ifdef HAVE_STRCASECMP
-#ifdef NEED_DECLARATION_STRCASECMP
-// Ultrix4.3's string.h fails to declare this.
-  int strcasecmp(const char *, const char *); }
-#endif /* NEED_DECLARATION_STRCASECMP */
-#else /* !HAVE_STRCASECMP */
-  int strcasecmp(const char *, const char *);
-#endif /* HAVE_STRCASECMP */
-
-#if !defined(_AIX) && !defined(sinix) && !defined(__sinix__)
-#ifdef HAVE_STRNCASECMP
-#ifdef NEED_DECLARATION_STRNCASECMP
-// SunOS's string.h fails to declare this.
-  int strncasecmp(const char *, const char *, int);
-#endif /* NEED_DECLARATION_STRNCASECMP */
-#else /* !HAVE_STRNCASECMP */
-  int strncasecmp(const char *, const char *, size_t);
-#endif /* HAVE_STRNCASECMP */
-#endif /* !_AIX && !sinix && !__sinix__ */
-
 #ifdef __cplusplus
 }
 #endif
