@@ -1,38 +1,29 @@
-// -*- C++ -*-
+/* Copyright 1989-2020 Free Software Foundation, Inc.
 
-// <groff_src_dir>/src/include/printer.h
+Written by James Clark (jjc@jclark.com)
 
-/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2002, 2003, 2004, 2006,
-                 2009
-   Free Software Foundation, Inc.
+This file is part of groff, the GNU roff typesetting system.
 
-   Written by James Clark (jjc@jclark.com)
+groff is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-   Last update: 5 Jan 2009
+groff is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
-   This file is part of groff.
-
-   groff is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-
-   groff is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program. If not, see <http://www.gnu.org/licenses/>.
-*/
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Description
 
-   The class `printer' performs the postprocessing.  Each
-   postprocessor only needs to implement a derived class of `printer' and
-   a suitable function `make_printer' for the device-dependent tasks.
-   Then the methods of class `printer' are called automatically by
-   `do_file()' in `input.cpp'.
+   The class 'printer' performs the postprocessing.  Each postprocessor
+   only needs to implement a derived class of 'printer' and a suitable
+   function 'make_printer' for the device-dependent tasks.  Then the
+   methods of class 'printer' are called automatically by 'do_file()' in
+   'input.cpp'.
 */
 
 #include "color.h"
@@ -69,9 +60,9 @@ public:
 			    int *, font **);
   font *get_font_from_index(int);
   virtual void draw(int, int *, int, const environment *);
-  // perform change of line color (text, outline) in the print-out 
+  // perform change of line color (text, outline) in the print-out
   virtual void change_color(const environment * const);
-  // perform change of fill color in the print-out 
+  // perform change of fill color in the print-out
   virtual void change_fill_color(const environment * const);
   virtual void begin_page(int) = 0;
   virtual void end_page(int) = 0;
@@ -99,3 +90,9 @@ private:
 };
 
 printer *make_printer();
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
