@@ -1,7 +1,7 @@
-/* Copyright (C) 2001-2018 Free Software Foundation, Inc.
+/* Copyright 2001-2020 Free Software Foundation, Inc.
      Written by Werner Lemberg (wl@gnu.org)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -16,18 +16,20 @@ for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
-
 /* This file is heavily based on the function __gen_tempname() in the
    file tempname.c which is part of the fileutils package. */
 
-
-#include "lib.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stddef.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <time.h>
 
+#include "lib.h"
+
 #include "posix.h"
 #include "nonposix.h"
 
@@ -78,10 +80,10 @@ int gen_tempname(char *tmpl, int dir)
   /* Get some more or less random data.  */
 #if HAVE_GETTIMEOFDAY
   timeval tv;
-  gettimeofday(&tv, NULL);
+  gettimeofday(&tv, 0);
   uint64_t random_time_bits = ((uint64_t)tv.tv_usec << 16) ^ tv.tv_sec;
 #else
-  uint64_t random_time_bits = time(NULL);
+  uint64_t random_time_bits = time(0);
 #endif
   value += random_time_bits ^ getpid();
 
@@ -115,3 +117,9 @@ int gen_tempname(char *tmpl, int dir)
   /* We got out of the loop because we ran out of combinations to try.  */
   return -1; /* EEXIST */
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
