@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.8.2.  */
+/* A Bison parser, made by GNU Bison 3.7.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -110,8 +110,10 @@ extern int yydebug;
     DELIM = 311,                   /* DELIM  */
     CHARTYPE = 312,                /* CHARTYPE  */
     SET = 313,                     /* SET  */
-    GRFONT = 314,                  /* GRFONT  */
-    GBFONT = 315                   /* GBFONT  */
+    RESET = 314,                   /* RESET  */
+    GRFONT = 315,                  /* GRFONT  */
+    GBFONT = 316,                  /* GBFONT  */
+    GIFONT = 317                   /* GIFONT  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -176,14 +178,16 @@ extern int yydebug;
 #define DELIM 311
 #define CHARTYPE 312
 #define SET 313
-#define GRFONT 314
-#define GBFONT 315
+#define RESET 314
+#define GRFONT 315
+#define GBFONT 316
+#define GIFONT 317
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 union YYSTYPE
 {
-#line 34 "../src/preproc/eqn/eqn.ypp"
+#line 35 "../src/preproc/eqn/eqn.ypp"
 
 	char *str;
 	box *b;
@@ -192,7 +196,7 @@ union YYSTYPE
 	int n;
 	column *col;
 
-#line 196 "src/preproc/eqn/eqn.hpp"
+#line 200 "src/preproc/eqn/eqn.hpp"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -203,8 +207,6 @@ typedef union YYSTYPE YYSTYPE;
 
 extern YYSTYPE yylval;
 
-
 int yyparse (void);
 
-
 #endif /* !YY_YY_SRC_PREPROC_EQN_EQN_HPP_INCLUDED  */
