@@ -1,8 +1,7 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2018 Free Software Foundation, Inc.
+/* Copyright 1989-2007 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -16,8 +15,15 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+
 // piles and matrices
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <assert.h>
+
 #include "eqn.h"
 #include "pbox.h"
 
@@ -34,16 +40,18 @@ int pile_box::compute_metrics(int style)
     printf(">?\\n[" WIDTH_FORMAT "]", col.p[i]->uid);
   printf("\n");
   printf(".nr " BASELINE_SEP_FORMAT " %dM",
-	 uid, baseline_sep+col.space);
+	 uid, get_param("baseline_sep") + col.space);
   for (i = 1; i < col.len; i++)
     printf(">?(\\n[" DEPTH_FORMAT "]+\\n[" HEIGHT_FORMAT "]+%dM)",
-	   col.p[i-1]->uid, col.p[i]->uid, default_rule_thickness*5);
-  // round it so that it's a multiple of the vertical resolution
+	   col.p[i-1]->uid, col.p[i]->uid,
+	   get_param("default_rule_thickness") * 5);
+  // round it so that it's a multiple of the vertical motion quantum
   printf("+(\\n(.V/2)/\\n(.V*\\n(.V\n");
 
   printf(".nr " SUP_RAISE_FORMAT " \\n[" BASELINE_SEP_FORMAT "]*%d/2"
 	 "+%dM\n",
-	 uid, uid, col.len-1, axis_height - shift_down);
+	 uid, uid, col.len-1, get_param("axis_height")
+			      - get_param("shift_down"));
   printf(".nr " HEIGHT_FORMAT " \\n[" SUP_RAISE_FORMAT "]+\\n["
 	 HEIGHT_FORMAT "]\n",
 	 uid, uid, col.p[0]->uid);
@@ -71,7 +79,7 @@ void pile_box::output()
 	       uid, col.p[i]->uid);
 	break;
       default:
-	assert(0);
+	assert(0 == "unhandled case of column alignment");
       }
       col.p[i]->output();
       printf("\\h'-\\n[" WIDTH_FORMAT "]u'", col.p[i]->uid);
@@ -87,7 +95,7 @@ void pile_box::output()
 	       col.p[i]->uid, uid);
 	break;
       default:
-	assert(0);
+	assert(0 == "unhandled case of column alignment");
       }
       if (i != col.len - 1)
 	printf("\\v'\\n[" BASELINE_SEP_FORMAT "]u'", uid);
@@ -109,7 +117,7 @@ void pile_box::output()
       av = "center";
       break;
     default:
-      assert(0);
+      assert(0 == "unhandled case of column alignment");
     }
     printf("<mtable columnalign='%s'>", av);
     for (int i = 0; i < col.len; i++) {
@@ -125,9 +133,9 @@ pile_box::pile_box(box *pp) : col(pp)
 {
 }
 
-void pile_box::check_tabs(int level)
+void pile_box::diagnose_tab_stop_usage(int level)
 {
-  col.list_check_tabs(level);
+  col.list_diagnose_tab_stop_usage(level);
 }
 
 void pile_box::debug_print()
@@ -155,21 +163,24 @@ int matrix_box::compute_metrics(int style)
     printf("\n");
   }
   printf(".nr " WIDTH_FORMAT " %dM",
-	 uid, column_sep*(len-1)+2*matrix_side_sep);
+	 uid, get_param("column_sep") * (len - 1) + 2
+	 * get_param("matrix_side_sep"));
   for (i = 0; i < len; i++)
     printf("+\\n[" COLUMN_WIDTH_FORMAT "]", uid, i);
   printf("\n");
   printf(".nr " BASELINE_SEP_FORMAT " %dM",
-	 uid, baseline_sep+space);
+	 uid, get_param("baseline_sep") + space);
   for (i = 0; i < len; i++)
     for (j = 1; j < p[i]->len; j++)
       printf(">?(\\n[" DEPTH_FORMAT "]+\\n[" HEIGHT_FORMAT "]+%dM)",
-	   p[i]->p[j-1]->uid, p[i]->p[j]->uid, default_rule_thickness*5);
-  // round it so that it's a multiple of the vertical resolution
+	   p[i]->p[j-1]->uid, p[i]->p[j]->uid,
+	   get_param("default_rule_thickness") * 5);
+  // round it so that it's a multiple of the vertical motion quantum
   printf("+(\\n(.V/2)/\\n(.V*\\n(.V\n");
   printf(".nr " SUP_RAISE_FORMAT " \\n[" BASELINE_SEP_FORMAT "]*%d/2"
 	 "+%dM\n",
-	 uid, uid, max_len-1, axis_height - shift_down);
+	 uid, uid, max_len-1, get_param("axis_height")
+	 - get_param("shift_down"));
   printf(".nr " HEIGHT_FORMAT " 0\\n[" SUP_RAISE_FORMAT "]+(0",
 	 uid, uid);
   for (i = 0; i < len; i++)
@@ -188,7 +199,7 @@ int matrix_box::compute_metrics(int style)
 void matrix_box::output()
 {
   if (output_format == troff) {
-    printf("\\h'%dM'", matrix_side_sep);
+    printf("\\h'%dM'", get_param("matrix_side_sep"));
     for (int i = 0; i < len; i++) {
       int j;
       printf("\\v'-\\n[" SUP_RAISE_FORMAT "]u'", uid);
@@ -205,7 +216,7 @@ void matrix_box::output()
 		 uid, i, p[i]->p[j]->uid);
 	  break;
 	default:
-	  assert(0);
+	  assert(0 == "unhandled case of column alignment");
 	}
 	p[i]->p[j]->output();
 	printf("\\h'-\\n[" WIDTH_FORMAT "]u'", p[i]->p[j]->uid);
@@ -221,7 +232,7 @@ void matrix_box::output()
 		 p[i]->p[j]->uid, uid, i);
 	  break;
 	default:
-	  assert(0);
+	  assert(0 == "unhandled case of column alignment");
 	}
 	if (j != p[i]->len - 1)
 	  printf("\\v'\\n[" BASELINE_SEP_FORMAT "]u'", uid);
@@ -230,9 +241,9 @@ void matrix_box::output()
       printf("\\v'-(%du*\\n[" BASELINE_SEP_FORMAT "]u)'", p[i]->len - 1, uid);
       printf("\\h'\\n[" COLUMN_WIDTH_FORMAT "]u'", uid, i);
       if (i != len - 1)
-	printf("\\h'%dM'", column_sep);
+	printf("\\h'%dM'", get_param("column_sep"));
     }
-    printf("\\h'%dM'", matrix_side_sep);
+    printf("\\h'%dM'", get_param("matrix_side_sep"));
   }
   else if (output_format == mathml) {
     int n = p[0]->len;	// Each column must have the same number of rows in it
@@ -252,7 +263,7 @@ void matrix_box::output()
 	  av = "center";
 	  break;
 	default:
-	  assert(0);
+	  assert(0 == "unhandled case of column alignment");
 	}
 	printf("<mtd columnalign='%s'>", av);
 	p[j]->p[i]->output();
@@ -278,7 +289,7 @@ matrix_box::~matrix_box()
 {
   for (int i = 0; i < len; i++)
     delete p[i];
-  a_delete p;
+  delete[] p;
 }
 
 void matrix_box::append(column *pp)
@@ -288,15 +299,15 @@ void matrix_box::append(column *pp)
     maxlen *= 2;
     p = new column*[maxlen];
     memcpy(p, oldp, sizeof(column*)*len);
-    a_delete oldp;
+    delete[] oldp;
   }
   p[len++] = pp;
 }
 
-void matrix_box::check_tabs(int level)
+void matrix_box::diagnose_tab_stop_usage(int level)
 {
   for (int i = 0; i < len; i++)
-    p[i]->list_check_tabs(level);
+    p[i]->list_diagnose_tab_stop_usage(level);
 }
 
 void matrix_box::debug_print()
@@ -338,10 +349,15 @@ void column::debug_print(const char *s)
     c = 'c';
     break;
   default:
-    assert(0);
+    assert(0 == "unhandled case of column alignment");
   }
   fprintf(stderr, "%c%s %d { ", c, s, space);
   list_debug_print(" above ");
   fprintf(stderr, " }");
 }
 
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
