@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.8.2.  */
+/* A Bison parser, made by GNU Bison 3.7.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -73,12 +73,12 @@ extern int yydebug;
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 union YYSTYPE
 {
-#line 218 "../src/preproc/refer/label.ypp"
+#line 222 "../src/preproc/refer/label.ypp"
 
   int num;
   expression *expr;
-  struct { int ndigits; int val; } dig;
-  struct { int start; int len; } str;
+  struct dig_s { int ndigits; int val; } dig;
+  struct str_s { int start; int len; } str;
 
 #line 84 "src/preproc/refer/label.hpp"
 
@@ -91,8 +91,6 @@ typedef union YYSTYPE YYSTYPE;
 
 extern YYSTYPE yylval;
 
-
 int yyparse (void);
 
-
 #endif /* !YY_YY_SRC_PREPROC_REFER_LABEL_HPP_INCLUDED  */
