@@ -1,8 +1,8 @@
 #!/bin/sh
 #
-# Copyright (C) 2020-2023 Free Software Foundation, Inc.
+# Copyright 2020-2023 G. Branden Robinson
 #
-# This file is part of groff.
+# This file is part of groff, the GNU roff typesetting system.
 #
 # groff is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -20,15 +20,28 @@
 
 groff="${abs_top_builddir:-.}/test-groff"
 
+fail=
+
+wail () {
+  echo "...FAILED" >&2
+  fail=yes
+}
+
 # Ensure that we get backtrace output across file and pipe boundaries.
 # Savannah #58153.
-OUT=$("$groff" -b -ww -U 2>&1 >/dev/null <<EOF
+output=$("$groff" -b -ww -U 2>&1 <<EOF
 .ec @
 .pso printf '@s[-20]'
 EOF
 )
 
-set -e
+echo "$output"
 
-printf "%s\n" "$OUT" | grep -qw 'backtrace: pipe'
-printf "%s\n" "$OUT" | grep -qw 'backtrace: file'
+echo "checking that pipe object is visible in backtrace"
+printf "%s\n" "$output" | grep -qw 'backtrace: pipe' || wail
+echo "checking that file object is visible in backtrace"
+printf "%s\n" "$output" | grep -qw 'backtrace: file' || wail
+
+test -z "$fail"
+
+# vim:set autoindent expandtab shiftwidth=4 tabstop=4 textwidth=72:
