@@ -1,18 +1,20 @@
-#! /bin/sh
+#!@POSIX_SHELL_PROG@
+#
 # Emulate nroff with groff.
 #
-# Copyright (C) 1992-2021 Free Software Foundation, Inc.
+# Copyright 1992-2021 Free Software Foundation, Inc.
+#           2019-2024 G. Branden Robinson
 #
 # Written by James Clark, Werner Lemberg, and G. Branden Robinson.
 #
-# This file is part of 'groff'.
+# This file is part of groff, the GNU roff typesetting system.
 #
-# 'groff' is free software; you can redistribute it and/or modify it
+# groff is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License (GPL) as published
 # by the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# 'groff' is distributed in the hope that it will be useful, but
+# groff is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
@@ -20,7 +22,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-prog="$0"
+# Screen out shells non-conforming with POSIX Issue 4 (1994).
+badshell=yes
+# Solaris 10 /bin/sh is so wretched that it not only doesn't support
+# standard parameter expansion, but it also writes diagnostic messages
+# to the standard output instead of standard error.
+if [ -n "$SHELL" ]
+then
+  "$SHELL" -c 'prog=${0##*/}' >/dev/null 2>&1 && badshell=
+fi
+
+if [ -n "$badshell" ]
+then
+  prog=`basename $0`
+else
+  prog=${0##*/}
+fi
 
 T=
 Topt=
@@ -28,7 +45,7 @@ opts=
 dry_run=
 is_option_argument_pending=
 
-usage="usage: $prog [-bcCEhikpRStUVz] [-d ctext] [-d string=text] \
+usage="usage: $prog [-abcCDEhiIkpRStUVzZ] [-d ctext] [-d string=text] \
 [-K fallback-encoding] [-m macro-package] [-M macro-directory] \
 [-n page-number] [-o page-list] [-P postprocessor-argument] \
 [-r cnumeric-expression] [-r register=numeric-expression] \
@@ -37,6 +54,46 @@ usage="usage: $prog [-bcCEhikpRStUVz] [-d ctext] [-d string=text] \
 usage: $prog {-v | --version}
 usage: $prog --help"
 
+summary="
+Format documents for terminal devices with groff(1).  See the nroff(1)
+manual page."
+
+# Break up option clusters into separate arguments.
+newargs=
+for arg
+do
+  thisarg=$arg
+  while :
+  do
+    case $thisarg in
+      -[abCEikpRStUvzZ])
+        newargs="$newargs $thisarg"
+        break
+        ;;
+      -[abCEikpRStUvzZ]*)
+        if test -n "$badshell"
+        then
+          # POSIX doesn't actually require $SHELL, but fortunately at
+          # least one craptastic non-conforming shell offers it.
+          echo "$prog: option cluster '$thisarg' not supported with" \
+            "POSIX-non-conforming shell '$SHELL'" >&2
+          exit 2
+        fi
+        remainder=${thisarg#-?}
+        thisarg=${thisarg%%$remainder}
+        newargs="$newargs $thisarg"
+        thisarg=-$remainder
+        ;;
+      *)
+        newargs="$newargs $thisarg"
+        break
+        ;;
+    esac
+  done
+done
+
+set -- $newargs
+
 for arg
 do
   if [ -n "$is_option_argument_pending" ]
@@ -47,6 +104,19 @@ do
     continue
   fi
 
+  # groff(1) options we don't support:
+  #
+  # -e because of historical clash in meaning.
+  # -s "
+  # -f because terminal devices don't support font families.
+  # -g because terminals don't do graphics.  (Some do, but grotty(1)
+  #    does not produce ReGIS or Sixel output.  Yet we support `-p`.)
+  # -G "
+  # -j "
+  # -l because terminal output is not suitable for a print spooler.
+  # -L "
+  # -N because we don't support -e.
+  # -X because gxditview(1) doesn't support terminal documents (why?).
   case $arg in
     -c)
       opts="$opts $arg -P-c" ;;
@@ -55,10 +125,10 @@ do
     -[eq] | -s*)
       # ignore these options
       ;;
-    -[dKmMnoPrTwW])
+    -[dDIKmMnoPrTwW])
       is_option_argument_pending=yes
       opts="$opts $arg" ;;
-    -[bCEikpRStUz] | -[dKMmrnoPwW]*)
+    -[abCEikpRStUzZ] | -[dDIKMmrnoPwW]*)
       opts="$opts $arg" ;;
     -T*)
       Topt=$arg ;;
@@ -67,7 +137,7 @@ do
       #
       # Solaris 2.2 through at least Solaris 9 'man' invokes
       # 'nroff -u0 ... | col -x'.  Ignore the -u0, since 'less' and
-      # 'more' can use the emboldening info.  But disable SGR, since
+      # 'more' interpret overstriking sequences.  But disable SGR, since
       # Solaris 'col' mishandles it.
       opts="$opts -P-c" ;;
     -V)
@@ -77,6 +147,7 @@ do
       opts="$opts $arg" ;;
     --help)
       echo "$usage"
+      echo "$summary"
       exit 0 ;;
     --)
       shift
@@ -101,7 +172,7 @@ fi
 
 # Determine the -T option.  Was a valid one specified?
 case "$Topt" in
-  -Tascii | -Tlatin1 | -Tutf8 | -Tcp1047)
+  -Tascii | -Tlatin1 | -Tutf8)
     T=$Topt ;;
 esac
 
@@ -110,7 +181,7 @@ if [ -z "$T" ]
 then
   Tenv=-T$GROFF_TYPESETTER
   case "$Tenv" in
-    -Tascii | -Tlatin1 | -Tutf8 | -Tcp1047)
+    -Tascii | -Tlatin1 | -Tutf8)
       T=$Tenv ;;
   esac
 fi
@@ -126,8 +197,6 @@ then
       Tloc=utf8 ;;
     ISO-8859-1 | ISO-8859-15)
       Tloc=latin1 ;;
-    IBM-1047)
-      Tloc=cp1047 ;;
     *)
       # Some old shells don't support ${FOO:-bar} expansion syntax.  We
       # should switch to it when it is safe to abandon support for them.
@@ -137,16 +206,12 @@ then
         iso_8859_1 | *.ISO-8859-1 | *.ISO8859-1 | \
         iso_8859_15 | *.ISO-8859-15 | *.ISO8859-15)
           Tloc=latin1 ;;
-        *.IBM-1047)
-          Tloc=cp1047 ;;
         *)
           case "$LESSCHARSET" in
             utf-8)
               Tloc=utf8 ;;
             latin1)
               Tloc=latin1 ;;
-            cp1047)
-              Tloc=cp1047 ;;
             *)
               Tloc=ascii ;;
           esac ;;
@@ -186,16 +251,11 @@ groff=${GROFF_TEST_GROFF-groff}
 #
 # If POSIX adopts Bash's ${var@Q} or an equivalent, this issue can be
 # revisited.
-#
-# Note 2: The construction '${1+"@$"}' preserves the absence of
-# arguments in old shells; see "Shell Substitutions" in the GNU Autoconf
-# manual.  We don't want 'nroff' to become 'groff ... ""' if $# equals
-# zero.
 if [ -n "$dry_run" ]
 then
-  echo PATH="$GROFF_RUNTIME$PATH" $groff $T $opts ${1+"$@"}
+  echo PATH="$GROFF_RUNTIME$PATH" $groff $T $opts "$@"
 else
-  PATH="$GROFF_RUNTIME$PATH" $groff $T $opts ${1+"$@"}
+  PATH="$GROFF_RUNTIME$PATH" $groff $T $opts "$@"
 fi
 
 # Local Variables:
