@@ -1,8 +1,7 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
+/* Copyright 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -539,7 +538,7 @@ void justification_spec::justify(output_line *col, vunits *bottomp) const
   assert(total == V0);
   *bottomp = height - gap;
 }
-  
+
 void column::justify(const justification_spec &js)
 {
   check_bottom();
@@ -547,26 +546,26 @@ void column::justify(const justification_spec &js)
   check_bottom();
 }
 
-void column_justify()
+static void column_justify()
 {
   vunits height;
   if (!the_column->is_active())
     error("can't justify column - column not active");
-  else if (get_vunits(&height, 'v')) {
+  else if (read_vunits(&height, 'v')) {
     justification_spec js(height);
-    symbol nm = get_long_name(true /* required */);
+    symbol nm = read_long_identifier(true /* required */);
     if (!nm.is_null()) {
       vunits v;
-      if (get_vunits(&v, 'v')) {
+      if (read_vunits(&v, 'v')) {
 	js.append(nm, v);
 	int err = 0;
 	while (has_arg()) {
-	  nm = get_long_name(true /* required */);
+	  nm = read_long_identifier(true /* required */);
 	  if (nm.is_null()) {
 	    err = 1;
 	    break;
 	  }
-	  if (!get_vunits(&v, 'v')) {
+	  if (!read_vunits(&v, 'v')) {
 	    err = 1;
 	    break;
 	  }
@@ -580,7 +579,7 @@ void column_justify()
   skip_line();
 }
 
-void column_start()
+static void column_start()
 {
   if (the_column->is_active())
     error("can't start column - column already active");
@@ -589,7 +588,7 @@ void column_start()
   skip_line();
 }
 
-void column_output()
+static void column_output()
 {
   if (!the_column->is_active())
     error("can't output column - column not active");
@@ -598,7 +597,7 @@ void column_output()
   skip_line();
 }
 
-void column_trim()
+static void column_trim()
 {
   if (!the_column->is_active())
     error("can't trim column - column not active");
@@ -607,7 +606,7 @@ void column_trim()
   skip_line();
 }
 
-void column_reset()
+static void column_reset()
 {
   if (!the_column->is_active())
     error("can't reset column - column not active");
@@ -652,7 +651,7 @@ class vjustify_node : public node {
   symbol typ;
 public:
   vjustify_node(symbol);
-  int reread(int *);
+  bool need_reread(bool *);
   const char *type();
   int same(node *);
   node *copy();
@@ -670,19 +669,19 @@ node *vjustify_node::copy()
 
 const char *vjustify_node::type()
 {
-  return "vjustify_node";
+  return "vertical justification node";
 }
 
 int vjustify_node::same(node *nd)
 {
-  return typ == ((vjustify_node *)nd)->typ;
+  return typ == static_cast<vjustify_node *>(nd)->typ;
 }
 
-int vjustify_node::reread(int *bolp)
+bool vjustify_node::need_reread(bool *bolp)
 {
   curdiv->vjustify(typ);
-  *bolp = 1;
-  return 1;
+  *bolp = true;
+  return true;
 }
 
 void macro_diversion::vjustify(symbol type)
@@ -699,13 +698,13 @@ void top_level_diversion::vjustify(symbol type)
   the_output->vjustify(vertical_position, type);
 }
 
-void no_vjustify()
+static void no_vjustify()
 {
   skip_line();
   no_vjustify_mode = 1;
 }
 
-void restore_vjustify()
+static void restore_vjustify()
 {
   skip_line();
   no_vjustify_mode = 0;
@@ -725,7 +724,13 @@ void init_column_requests()
   register_dictionary.define(".colx", new column_extra_space_reg);
   register_dictionary.define(".cola", new column_active_reg);
   register_dictionary.define(".nvj",
-			       new readonly_register(&no_vjustify_mode));
+			     new readonly_register(&no_vjustify_mode));
 }
 
 #endif /* COLUMN */
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
