@@ -1,7 +1,7 @@
-/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
+/* Copyright 1989-2025 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -24,7 +24,7 @@ struct size_range {
 };
 
 class font_size {
-  static size_range *size_table;
+  static size_range *size_list;
   static int nranges;
   int p;
 public:
@@ -35,7 +35,8 @@ public:
   int to_units();
   int operator==(font_size);
   int operator!=(font_size);
-  static void init_size_table(int *sizes);
+  static void init_size_list(int * /* sizes */);
+  static void dump_size_list();
 };
 
 inline font_size::font_size() : p(0)
@@ -59,7 +60,7 @@ inline int font_size::to_scaled_points()
 
 inline int font_size::to_points()
 {
-  return p/sizescale;
+  return (p / sizescale);
 }
 
 class environment;
@@ -80,20 +81,21 @@ class tab_stops {
   tab *repeated_list;
 public:
   tab_stops();
-  tab_stops(hunits distance, tab_type type);
+  tab_stops(hunits /* distance */, tab_type /* type */);
   tab_stops(const tab_stops &);
   ~tab_stops();
   void operator=(const tab_stops &);
-  tab_type distance_to_next_tab(hunits pos, hunits *distance);
-  tab_type distance_to_next_tab(hunits curpos, hunits *distance, hunits *leftpos);
+  tab_type distance_to_next_tab(hunits /* pos */,
+				hunits * /* distance */);
+  tab_type distance_to_next_tab(hunits /* curpos */,
+				hunits * /* distance */,
+				hunits * /* leftpos */);
   void clear();
-  void add_tab(hunits pos, tab_type type, int repeated);
+  void add_tab(hunits /* pos */, tab_type /* type */,
+	       bool /* is_repeated */);
   const char *to_string();
 };
 
-const unsigned MARGIN_CHARACTER_ON = 1;
-const unsigned MARGIN_CHARACTER_NEXT = 2;
-
 class charinfo;
 struct node;
 struct breakpoint;
@@ -115,8 +117,8 @@ void line_spacing();
 void line_length();
 void indent();
 void temporary_indent();
-void do_underline(int);
-void do_input_trap(int);
+void configure_underlining(bool);
+void do_input_trap(bool);
 void set_tabs();
 void margin_character();
 void no_number();
@@ -140,7 +142,7 @@ void widow_control_request();
 #endif /* WIDOW_CONTROL */
 
 class environment {
-  int dummy;			// dummy environment used for \w
+  bool is_dummy_env;			// dummy environment used for \w
   hunits prev_line_length;
   hunits line_length;
   hunits prev_title_length;
@@ -158,11 +160,11 @@ class environment {
   int space_size;		// in 36ths of an em
   int sentence_space_size;	// same but for spaces at the end of sentences
   int adjust_mode;
-  int fill;
-  int interrupted;
-  int prev_line_interrupted;
-  int center_lines;
-  int right_justify_lines;
+  bool is_filling;
+  bool was_line_interrupted;
+  int was_previous_line_interrupted;	// three-valued Boolean :-|
+  int centered_line_count;
+  int right_aligned_line_count;
   vunits prev_vertical_spacing;
   vunits vertical_spacing;
   vunits prev_post_vertical_spacing;
@@ -172,15 +174,15 @@ class environment {
   hunits prev_indent;
   hunits indent;
   hunits temporary_indent;
-  int have_temporary_indent;
+  bool have_temporary_indent;
   hunits saved_indent;
   hunits target_text_length;
   int pre_underline_fontno;
-  int underline_lines;
-  int underline_spaces;
+  int underlined_line_count;
+  bool underline_spaces;
   symbol input_trap;
   int input_trap_count;
-  int continued_input_trap;
+  bool continued_input_trap;
   node *line;			// in reverse order
   hunits prev_text_length;
   hunits width_total;
@@ -189,20 +191,20 @@ class environment {
   node *tab_contents;
   hunits tab_width;
   hunits tab_distance;
-  int line_tabs;
+  bool using_line_tabs;
   tab_type current_tab;
   node *leader_node;
   charinfo *tab_char;
   charinfo *leader_char;
-  int current_field;		// is there a current field?
+  bool has_current_field;
   hunits field_distance;
   hunits pre_field_width;
   int field_spaces;
   int tab_field_spaces;
-  int tab_precedes_field;
-  int discarding;
-  int spread_flag;		// set by \p
-  unsigned margin_character_flags;
+  bool tab_precedes_field;
+  bool is_discarding;
+  bool is_spreading;		// set by \p
+  unsigned char margin_character_flags;
   node *margin_character_node;
   hunits margin_character_distance;
   node *numbering_nodes;
@@ -211,60 +213,75 @@ class environment {
   int line_number_indent;	// in digit spaces
   int line_number_multiple;
   int no_number_count;
-  unsigned hyphenation_flags;
+  unsigned int hyphenation_mode;
+  unsigned int hyphenation_mode_default;
   int hyphen_line_count;
   int hyphen_line_max;
   hunits hyphenation_space;
   hunits hyphenation_margin;
-  int composite;		// used for construction of composite char?
+  bool composite;	// used for construction of composite character
   pending_output_line *pending_lines;
 #ifdef WIDOW_CONTROL
-  int widow_control;
+  bool want_widow_control;
 #endif /* WIDOW_CONTROL */
-  color *glyph_color;
-  color *prev_glyph_color;
+  color *stroke_color;
+  color *prev_stroke_color;
   color *fill_color;
   color *prev_fill_color;
+  unsigned char control_character;
+  unsigned char no_break_control_character;
 
   tab_type distance_to_next_tab(hunits *);
-  tab_type distance_to_next_tab(hunits *distance, hunits *leftpos);
+  tab_type distance_to_next_tab(hunits * /* distance */,
+				hunits * /* leftpos */);
   void start_line();
-  void output_line(node *, hunits, int);
-  void output(node *nd, int retain_size, vunits vs, vunits post_vs,
-	      hunits width, int was_centered);
-  void output_title(node *nd, int retain_size, vunits vs, vunits post_vs,
-		    hunits width);
+  void output_line(node * /* nd */, hunits /* width */,
+		   bool /* was_centered */);
+  void output(node * /* nd */, bool /* suppress_filling */,
+	      vunits /* vs */, vunits /* post_vs */, hunits /* width */,
+	      bool /* was_centered */);
+  void output_title(node * /* nd */, bool /* suppress_filling */,
+		    vunits /* vs */, vunits /* post_vs */,
+		    hunits /* width */);
 #ifdef WIDOW_CONTROL
   void mark_last_line();
 #endif /* WIDOW_CONTROL */
   breakpoint *choose_breakpoint();
-  void hyphenate_line(int start_here = 0);
+  void possibly_hyphenate_line(bool /* must_break_here */ = false);
   void start_field();
   void wrap_up_field();
   void add_padding();
-  node *make_tab_node(hunits d, node *next = 0);
+  node *make_tab_node(hunits /* d */,
+		      node * /* next */ = 0 /* nullptr */);
   node *get_prev_char();
 public:
-  int seen_space;
-  int seen_eol;
-  int suppress_next_eol;
-  int seen_break;
+  // C++11: Use `enum : unsigned char`.
+  enum {
+    MC_ON = 0x1,
+    MC_NEXT = 0x2
+  };
+  bool seen_space;
+  bool seen_eol;
+  bool suppress_next_eol;
+  bool seen_break;
   tab_stops tabs;
   const symbol name;
-  unsigned char control_char;
-  unsigned char no_break_control_char;
   charinfo *hyphen_indicator_char;
-  
+
   environment(symbol);
   environment(const environment *);	// for temporary environment
   ~environment();
-  statem *construct_state(int only_eol);
-  void print_env();
+  unsigned char get_control_character();
+  bool set_control_character(unsigned char);
+  unsigned char get_no_break_control_character();
+  bool set_no_break_control_character(unsigned char);
+  statem *construct_state(bool has_only_eol);
+  void dump();
   void copy(const environment *);
-  int is_dummy() { return dummy; }
-  int is_empty();
-  int is_composite() { return composite; }
-  void set_composite() { composite = 1; }
+  bool is_dummy() { return is_dummy_env; }
+  bool is_empty();
+  bool is_composite() { return composite; }
+  void set_composite() { composite = true; }
   vunits get_vertical_spacing();	// .v
   vunits get_post_vertical_spacing();	// .pvs
   int get_line_spacing();		// .L
@@ -280,8 +297,8 @@ public:
   int get_zoom();			// .zoom
   int get_numbering_nodes();		// .nm
   font_family *get_family() { return family; }
-  int get_bold();			// .b
-  int get_adjust_mode();		// .j
+  hunits get_emboldening_offset();	// .b
+  unsigned get_adjust_mode();		// .j
   int get_fill();			// .u
   hunits get_indent();			// .i
   hunits get_temporary_indent();
@@ -303,21 +320,26 @@ public:
     { return env_half_narrow_space_width(this); }
   hunits get_input_line_position();
   const char *get_tabs();
-  int get_line_tabs();
-  int get_hyphenation_flags();
+  int is_using_line_tabs();
+  unsigned get_hyphenation_mode();
+  unsigned get_hyphenation_mode_default();
   int get_hyphen_line_max();
   int get_hyphen_line_count();
   hunits get_hyphenation_space();
   hunits get_hyphenation_margin();
-  int get_center_lines();
-  int get_right_justify_lines();
+  int get_centered_line_count();
+  int get_input_trap_line_count();
+  int get_input_trap_respects_continuation();
+  const char *get_input_trap_macro();
+  int get_right_aligned_line_count();
   int get_no_number_count();
-  int get_prev_line_interrupted() { return prev_line_interrupted; }
+  bool get_was_line_interrupted() { return was_line_interrupted; }
+  int get_was_previous_line_interrupted() { return was_previous_line_interrupted; }
   color *get_fill_color();
-  color *get_glyph_color();
-  color *get_prev_glyph_color();
+  color *get_stroke_color();
+  color *get_prev_stroke_color();
   color *get_prev_fill_color();
-  void set_glyph_color(color *c);
+  void set_stroke_color(color *c);
   void set_fill_color(color *c);
   node *make_char_node(charinfo *);
   node *extract_output_line();
@@ -331,13 +353,14 @@ public:
   void set_char_slant(int);
   void set_input_line_position(hunits);	// used by \n(hp
   void interrupt();
-  void spread() { spread_flag = 1; }
-  void possibly_break_line(int start_here = 0, int forced = 0);
-  void do_break(int spread = 0);	// .br
+  void spread() { is_spreading = true; }
+  void possibly_break_line(bool /* must_break_here */ = false,
+			   bool /* must_adjust */ = false);
+  void do_break(bool /* want_adjustment */ = false);	// .br, .brp
   void final_break();
   node *make_tag(const char *name, int i);
   void newline();
-  void handle_tab(int is_leader = 0);	// do a tab or leader
+  void advance_to_tab_stop(bool /* use_leader */ = false);
   void add_node(node *);
   void add_char(charinfo *);
   void add_hyphen_indicator();
@@ -345,8 +368,10 @@ public:
   void space();
   void space(hunits, hunits);
   void space_newline();
-  const char *get_glyph_color_string();
+  const char *get_stroke_color_string();
   const char *get_fill_color_string();
+  const char *get_prev_stroke_color_string();
+  const char *get_prev_fill_color_string();
   const char *get_font_family_string();
   const char *get_font_name_string();
   const char *get_style_name_string();
@@ -354,10 +379,12 @@ public:
   const char *get_point_size_string();
   const char *get_requested_point_size_string();
   void output_pending_lines();
-  void construct_format_state(node *n, int was_centered, int fill);
+  void construct_format_state(node * /* nd */, bool /* was_centered */,
+			      int /* fill */);
   void construct_new_line_state(node *n);
   void dump_troff_state();
-  
+  void dump_pending_nodes();
+
   friend void title_length();
   friend void space_size();
   friend void fill();
@@ -372,15 +399,16 @@ public:
   friend void line_length();
   friend void indent();
   friend void temporary_indent();
-  friend void do_underline(int);
-  friend void do_input_trap(int);
+  friend void configure_underlining(bool);
+  friend void do_input_trap(bool);
   friend void set_tabs();
   friend void margin_character();
   friend void no_number();
   friend void number_lines();
-  friend void leader_character();
-  friend void tab_character();
+  friend void leader_character_request();
+  friend void tab_character_request();
   friend void hyphenate_request();
+  friend void set_hyphenation_mode_default();
   friend void no_hyphenate();
   friend void hyphen_line_max_request();
   friend void hyphenation_space_request();
@@ -395,22 +423,20 @@ public:
 #ifdef WIDOW_CONTROL
   friend void widow_control_request();
 #endif /* WIDOW_CONTROL */
-
-  friend void do_divert(int append, int boxing);
+  friend void do_divert(bool /* appending */, bool /* boxing */);
 };
 
 extern environment *curenv;
 extern void pop_env();
 extern void push_env(int);
 
+void select_font(symbol);
 void init_environments();
-void read_hyphen_file(const char *name);
 
 extern double spread_limit;
 
-extern int break_flag;
 extern symbol default_family;
-extern int translate_space_to_dummy;
+extern bool translate_space_to_dummy;
 
 extern unsigned char hpf_code_table[];
 
