@@ -1,8 +1,7 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2018 Free Software Foundation, Inc.
+/* Copyright 1989-2025 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -18,16 +17,19 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 struct hyphen_list {
-  unsigned char hyphen;
-  unsigned char breakable;
+  bool is_hyphen;
+  bool is_breakable;
   unsigned char hyphenation_code;
   hyphen_list *next;
-  hyphen_list(unsigned char code, hyphen_list *p = 0);
+  hyphen_list(unsigned char code,
+	      hyphen_list * /* p */ = 0 /* nullptr */);
 };
 
-void hyphenate(hyphen_list *, unsigned);
-
-enum hyphenation_type { HYPHEN_MIDDLE, HYPHEN_BOUNDARY, HYPHEN_INHIBIT };
+enum hyphenation_type {
+  HYPHENATION_PERMITTED,
+  HYPHENATION_UNNECESSARY,
+  HYPHENATION_INHIBITED
+};
 
 class ascii_output_file;
 
@@ -51,25 +53,30 @@ struct node {
   statem *state;
   statem *push_state;
   int div_nest_level;
-  int is_special;
+  bool is_special;
   node();
   node(node *);
   node(node *, statem *, int);
-  node *add_char(charinfo *, environment *, hunits *, int *, node ** = 0);
+  node(node *, statem *, int, bool);
+  virtual node *add_char(charinfo *, environment *, hunits *, int *,
+		 node ** /* glyph_comp_np */ = 0 /* nullptr */);
 
   virtual ~node();
   virtual node *copy() = 0;
-  virtual int set_unformat_flag();
-  virtual int force_tprint() = 0;
-  virtual int is_tag() = 0;
-  virtual int get_break_code();
+  virtual bool set_unformat_flag();
+  virtual bool causes_tprint() = 0;
+  virtual bool is_tag() = 0;
+  // TODO: Figure out what a hyphenation code means in the UTF-8 future,
+  // where a "grochar" is a vector of NFD decomposed code points.  Can
+  // it be a scalar--a 32-bit int?
+  virtual unsigned char get_break_code();
   virtual hunits width();
   virtual hunits subscript_correction();
   virtual hunits italic_correction();
   virtual hunits left_italic_correction();
   virtual hunits skew();
   virtual int nspaces();
-  virtual int merge_space(hunits, hunits, hunits);
+  virtual bool did_space_merge(hunits, hunits, hunits);
   virtual vunits vertical_width();
   virtual node *last_char_node();
   virtual void vertical_extent(vunits *, vunits *);
@@ -81,66 +88,85 @@ struct node {
   virtual node *add_self(node *, hyphen_list **);
   virtual hyphen_list *get_hyphen_list(hyphen_list *, int *);
   virtual void ascii_print(ascii_output_file *);
-  virtual void asciify(macro *);
-  virtual int discardable();
+  virtual void asciify(macro *) = 0;
+  virtual bool discardable();
   virtual void spread_space(int *, hunits *);
   virtual void freeze_space();
   virtual void is_escape_colon();
-  virtual breakpoint *get_breakpoints(hunits, int, breakpoint * = 0, int = 0);
+  virtual breakpoint *get_breakpoints(hunits, int,
+			      breakpoint * /* rest */ = 0 /* nullptr */,
+			      bool /* is_inner */ = false);
   virtual int nbreaks();
   virtual void split(int, node **, node **);
   virtual hyphenation_type get_hyphenation_type();
-  virtual int reread(int *);
+  virtual bool need_reread(bool *);
   virtual token_node *get_token_node();
-  virtual int overlaps_vertically();
-  virtual int overlaps_horizontally();
+  virtual bool overlaps_vertically();
+  virtual bool overlaps_horizontally();
   virtual units size();
-  virtual int interpret(macro *);
+  virtual bool interpret(macro *);
 
   virtual node *merge_glyph_node(glyph_node *);
   virtual tfont *get_tfont();
-  virtual color *get_glyph_color();
+  virtual color *get_stroke_color();
   virtual color *get_fill_color();
   virtual void tprint(troff_output_file *);
   virtual void zero_width_tprint(troff_output_file *);
 
-  node *add_italic_correction(hunits *);
+  virtual node *add_italic_correction(hunits *);
 
-  virtual int same(node *) = 0;
+  virtual bool is_same_as(node *) = 0;
   virtual const char *type() = 0;
-  virtual void debug_node();
-  virtual void debug_node_list();
+  virtual void dump_properties();
+  virtual void dump_node();
 };
 
 inline node::node()
-: next(0), last(0), state(0), push_state(0), div_nest_level(0), is_special(0)
+: next(0 /* nullptr */), last(0 /* nullptr */),
+  state(0 /* nullptr */), push_state(0 /* nullptr */),
+  div_nest_level(0), is_special(false)
 {
 }
 
 inline node::node(node *n)
-: next(n), last(0), state(0), push_state(0), div_nest_level(0), is_special(0)
+: next(n), last(0 /* nullptr */),
+  state(0 /* nullptr */), push_state(0 /* nullptr */),
+  div_nest_level(0), is_special(false)
 {
 }
 
 inline node::node(node *n, statem *s, int divlevel)
-: next(n), last(0), push_state(0), div_nest_level(divlevel), is_special(0)
+: next(n), last(0 /* nullptr */),
+  push_state(0 /* nullptr */),
+  div_nest_level(divlevel), is_special(false)
 {
-  if (s)
+  if (s != 0 /* nullptr */)
     state = new statem(s);
   else
-    state = 0;
+    state = 0 /* nullptr */;
+}
+
+inline node::node(node *n, statem *s, int divlevel, bool special)
+: next(n), last(0 /* nullptr */),
+  push_state(0 /* nullptr */),
+  div_nest_level(divlevel), is_special(special)
+{
+  if (s != 0 /* nullptr */)
+    state = new statem(s);
+  else
+    state = 0 /* nullptr */;
 }
 
 inline node::~node()
 {
-  if (state != 0)
+  if (state != 0 /* nullptr */)
     delete state;
-  if (push_state != 0)
+  if (push_state != 0 /* nullptr */)
     delete push_state;
 }
 
+// three-valued Boolean :-|
 // 0 means it doesn't, 1 means it does, 2 means it's transparent
-
 int node_list_ends_sentence(node *);
 
 struct breakpoint {
@@ -155,43 +181,48 @@ struct breakpoint {
 class line_start_node : public node {
 public:
   line_start_node() {}
-  node *copy() { return new line_start_node; }
-  int same(node *);
-  int force_tprint();
-  int is_tag();
-  const char *type();
   void asciify(macro *);
+  node *copy() { return new line_start_node; }
+  bool is_same_as(node *);
+  bool causes_tprint();
+  bool is_tag();
+  const char *type();
 };
 
+// Represent a space of any width, including zero.
 class space_node : public node {
 private:
 protected:
   hunits n;
-  char set;
-  char was_escape_colon;
+  bool set;
+  bool was_escape_colon;
   color *col;			/* for grotty */
-  space_node(hunits, int, int, color *, statem *, int, node * = 0);
+  space_node(hunits, int, int, color *, statem *, int,
+	     node * = 0 /* nullptr */);
 public:
-  space_node(hunits, color *, node * = 0);
+  space_node(hunits, color *, node * = 0 /* nullptr */);
+  void asciify(macro *);
   node *copy();
   int nspaces();
   hunits width();
-  int discardable();
-  int merge_space(hunits, hunits, hunits);
+  bool discardable();
+  bool did_space_merge(hunits, hunits, hunits);
   void freeze_space();
   void is_escape_colon();
   void spread_space(int *, hunits *);
   void tprint(troff_output_file *);
-  breakpoint *get_breakpoints(hunits, int, breakpoint * = 0, int = 0);
+  breakpoint *get_breakpoints(hunits, int,
+			      breakpoint * /* rest */ = 0 /* nullptr */,
+			      bool /* is_inner */ = false);
   int nbreaks();
   void split(int, node **, node **);
   void ascii_print(ascii_output_file *);
-  int same(node *);
-  void asciify(macro *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   hyphenation_type get_hyphenation_type();
+  void dump_properties();
 };
 
 struct width_list {
@@ -200,45 +231,54 @@ struct width_list {
   width_list *next;
   width_list(hunits, hunits);
   width_list(width_list *);
+  void dump();
 };
 
 class word_space_node : public space_node {
 protected:
   width_list *orig_width;
-  unsigned char unformat;
-  word_space_node(hunits, int, color *, width_list *, int, statem *, int,
-		  node * = 0);
+  bool unformat;
+  word_space_node(hunits /* d */, int /* s */, color * /* c */,
+		  width_list * /* w */, bool /* flag */,
+		  statem * /* st */, int /* divlevel */,
+		  node * /* x */ = 0 /* nullptr */);
 public:
-  word_space_node(hunits, color *, width_list *, node * = 0);
+  word_space_node(hunits, color *, width_list *,
+		  node * /* x */ = 0 /* nullptr */);
   ~word_space_node();
-  node *copy();
-  int reread(int *);
-  int set_unformat_flag();
-  void tprint(troff_output_file *);
-  int same(node *);
   void asciify(macro *);
+  node *copy();
+  bool need_reread(bool *);
+  bool set_unformat_flag();
+  void tprint(troff_output_file *);
+  bool is_same_as(node *);
   const char *type();
-  int merge_space(hunits, hunits, hunits);
-  int force_tprint();
-  int is_tag();
+  bool did_space_merge(hunits, hunits, hunits);
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
 class unbreakable_space_node : public word_space_node {
-  unbreakable_space_node(hunits, int, color *, statem *, int, node * = 0);
+  unbreakable_space_node(hunits, int, color *, statem *, int,
+			 node * /* x */ = 0 /* nullptr */);
 public:
-  unbreakable_space_node(hunits, color *, node * = 0);
-  node *copy();
-  int reread(int *);
-  void tprint(troff_output_file *);
-  int same(node *);
+  unbreakable_space_node(hunits, color *,
+			 node * /* x */ = 0 /* nullptr */);
   void asciify(macro *);
+  node *copy();
+  bool need_reread(bool *);
+  void tprint(troff_output_file *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
-  breakpoint *get_breakpoints(hunits, int, breakpoint * = 0, int = 0);
+  bool causes_tprint();
+  bool is_tag();
+  breakpoint *get_breakpoints(hunits, int,
+			      breakpoint * /* rest */ = 0 /* nullptr */,
+			      bool /* is_inner */ = false);
   int nbreaks();
   void split(int, node **, node **);
-  int merge_space(hunits, hunits, hunits);
+  bool did_space_merge(hunits, hunits, hunits);
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
@@ -247,28 +287,34 @@ public:
 class diverted_space_node : public node {
 public:
   vunits n;
-  diverted_space_node(vunits, node * = 0);
-  diverted_space_node(vunits, statem *, int, node * = 0);
+  diverted_space_node(vunits, node * /* p */ = 0 /* nullptr */);
+  diverted_space_node(vunits, statem *, int,
+		      node * /* p */ = 0 /* nullptr */);
+  void asciify(macro *);
   node *copy();
-  int reread(int *);
-  int same(node *);
+  bool need_reread(bool *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
 class diverted_copy_file_node : public node {
   symbol filename;
 public:
   vunits n;
-  diverted_copy_file_node(symbol, node * = 0);
-  diverted_copy_file_node(symbol, statem *, int, node * = 0);
+  diverted_copy_file_node(symbol, node * /* p */ = 0 /* nullptr */);
+  diverted_copy_file_node(symbol, statem *, int,
+			  node * /* p */ = 0 /* nullptr */);
+  void asciify(macro *);
   node *copy();
-  int reread(int *);
-  int same(node *);
+  bool need_reread(bool *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
 class extra_size_node : public node {
@@ -277,11 +323,13 @@ public:
   extra_size_node(vunits);
   extra_size_node(vunits, statem *, int);
   void set_vertical_size(vertical_size *);
+  void asciify(macro *);
   node *copy();
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
 class vertical_size_node : public node {
@@ -289,61 +337,68 @@ class vertical_size_node : public node {
 public:
   vertical_size_node(vunits, statem *, int);
   vertical_size_node(vunits);
-  void set_vertical_size(vertical_size *);
   void asciify(macro *);
+  void set_vertical_size(vertical_size *);
   node *copy();
-  int set_unformat_flag();
-  int same(node *);
+  bool set_unformat_flag();
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
 class hmotion_node : public node {
 protected:
   hunits n;
-  unsigned char was_tab;
-  unsigned char unformat;
+  bool was_tab;			// needed by `unformat`
+  bool unformat;
   color *col;			/* for grotty */
 public:
-  hmotion_node(hunits i, color *c, node *nxt = 0)
-    : node(nxt), n(i), was_tab(0), unformat(0), col(c) {}
-  hmotion_node(hunits i, color *c, statem *s, int divlevel, node *nxt = 0)
-    : node(nxt, s, divlevel), n(i), was_tab(0), unformat(0), col(c) {}
-  hmotion_node(hunits i, int flag1, int flag2, color *c, statem *s,
-	       int divlevel, node *nxt = 0)
+  hmotion_node(hunits i, color *c, node *nxt = 0 /* nullptr */)
+    : node(nxt), n(i), was_tab(false), unformat(false), col(c) {}
+  hmotion_node(hunits i, color *c, statem *s, int divlevel,
+	       node *nxt = 0 /* nullptr */)
+    : node(nxt, s, divlevel), n(i), was_tab(false), unformat(false),
+      col(c) {}
+  hmotion_node(hunits i, bool flag1, bool flag2, color *c, statem *s,
+	       int divlevel, node *nxt = 0 /* nullptr */)
     : node(nxt, s, divlevel), n(i), was_tab(flag1), unformat(flag2),
       col(c) {}
-  hmotion_node(hunits i, int flag1, int flag2, color *c, node *nxt = 0)
+  hmotion_node(hunits i, bool flag1, bool flag2, color *c,
+	       node *nxt = 0 /* nullptr */)
     : node(nxt), n(i), was_tab(flag1), unformat(flag2), col(c) {}
-  node *copy();
-  int reread(int *);
-  int set_unformat_flag();
   void asciify(macro *);
+  node *copy();
+  bool need_reread(bool *);
+  bool set_unformat_flag();
   void tprint(troff_output_file *);
   hunits width();
   void ascii_print(ascii_output_file *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
+  void dump_properties();
 };
 
 class space_char_hmotion_node : public hmotion_node {
 public:
-  space_char_hmotion_node(hunits, color *, node * = 0);
-  space_char_hmotion_node(hunits, color *, statem *, int, node * = 0);
+  space_char_hmotion_node(hunits, color *,
+			  node * /* nxt */ = 0 /* nullptr */);
+  space_char_hmotion_node(hunits, color *, statem *, int,
+			  node * /* nxt */ = 0 /* nullptr */);
   node *copy();
-  void ascii_print(ascii_output_file *);
   void asciify(macro *);
+  void ascii_print(ascii_output_file *);
   void tprint(troff_output_file *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
@@ -355,110 +410,129 @@ class vmotion_node : public node {
 public:
   vmotion_node(vunits, color *);
   vmotion_node(vunits, color *, statem *, int);
+  void asciify(macro *);
   void tprint(troff_output_file *);
   node *copy();
   vunits vertical_width();
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
-class hline_node : public node {
+struct container_node : public node {
+  node *nodes;
+  container_node(node * /* contents */);
+  container_node(node * /* next */, node * /* contents */);
+  container_node(node * /* next */, statem *, int);
+  //container_node(node * /* next */, statem *, node * /* contents */);
+  container_node(node * /* next */, statem *, int,
+		 node * /* contents */);
+  container_node(node * /* next */, statem *, int, bool,
+		 node * /* contents */);
+  ~container_node();
+  virtual void dump_node();
+};
+
+class hline_node : public container_node {
   hunits x;
-  node *n;
 public:
-  hline_node(hunits, node *, node * = 0);
-  hline_node(hunits, node *, statem *, int, node * = 0);
-  ~hline_node();
+  hline_node(hunits, node *, node * /* nxt */ = 0 /* nullptr */);
+  hline_node(hunits, node *, statem *, int,
+	     node * /* nxt */ = 0 /* nullptr */);
+  void asciify(macro *);
   node *copy();
   hunits width();
   void tprint(troff_output_file *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
-class vline_node : public node {
+class vline_node : public container_node {
   vunits x;
-  node *n;
 public:
-  vline_node(vunits, node *, node * = 0);
-  vline_node(vunits, node *, statem *, int, node * = 0);
-  ~vline_node();
+  vline_node(vunits, node *, node * /* nxt */ = 0 /* nullptr */);
+  vline_node(vunits, node *, statem *, int,
+	     node * /* nxt */ = 0 /* nullptr */);
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
   hunits width();
   vunits vertical_width();
   void vertical_extent(vunits *, vunits *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
 class dummy_node : public node {
 public:
-  dummy_node(node *nd = 0) : node(nd) {}
+  dummy_node(node * nd = 0 /* nullptr */) : node(nd) {}
+  void asciify(macro *);
   node *copy();
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   hyphenation_type get_hyphenation_type();
 };
 
 class transparent_dummy_node : public node {
 public:
-  transparent_dummy_node(node *nd = 0) : node(nd) {}
+  transparent_dummy_node(node * nd = 0 /* nullptr */) : node(nd) {}
+  void asciify(macro *);
   node *copy();
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   int ends_sentence();
   hyphenation_type get_hyphenation_type();
 };
 
-class zero_width_node : public node {
-  node *n;
+class zero_width_node : public container_node {
 public:
   zero_width_node(node *);
   zero_width_node(node *, statem *, int);
-  ~zero_width_node();
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
-  int same(node *);
+  void ascii_print(ascii_output_file *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   void append(node *);
   int character_type();
   void vertical_extent(vunits *, vunits *);
 };
 
-class left_italic_corrected_node : public node {
-  node *n;
+class left_italic_corrected_node : public container_node {
   hunits x;
 public:
-  left_italic_corrected_node(node * = 0);
-  left_italic_corrected_node(statem *, int, node * = 0);
-  ~left_italic_corrected_node();
+  left_italic_corrected_node(node * /* xx */ = 0 /* nullptr */);
+  left_italic_corrected_node(statem *, int,
+			     node * /* xx */ = 0 /* nullptr */);
+  void asciify(macro *);
   void tprint(troff_output_file *);
   void ascii_print(ascii_output_file *);
-  void asciify(macro *);
   node *copy();
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   hunits width();
   node *last_char_node();
   void vertical_extent(vunits *, vunits *);
   int ends_sentence();
-  int overlaps_horizontally();
-  int overlaps_vertically();
+  bool overlaps_horizontally();
+  bool overlaps_vertically();
   hyphenation_type get_hyphenation_type();
   tfont *get_tfont();
   int character_type();
@@ -468,71 +542,80 @@ public:
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   node *add_self(node *, hyphen_list **);
   node *merge_glyph_node(glyph_node *);
+  void dump_properties();
 };
 
-class overstrike_node : public node {
-  node *list;
+class overstrike_node : public container_node {
   hunits max_width;
 public:
   overstrike_node();
   overstrike_node(statem *, int);
-  ~overstrike_node();
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
   void overstrike(node *);	// add another node to be overstruck
   hunits width();
-  int same(node *);
+  void ascii_print(ascii_output_file *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
+  void dump_properties();
 };
 
-class bracket_node : public node {
-  node *list;
+class bracket_node : public container_node {
   hunits max_width;
 public:
   bracket_node();
   bracket_node(statem *, int);
-  ~bracket_node();
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
-  void bracket(node *);	// add another node to be overstruck
+  void bracket(node *);		// add another node to be stacked
   hunits width();
-  int same(node *);
+  void ascii_print(ascii_output_file *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
-class special_node : public node {
+class device_extension_node : public node {
   macro mac;
   tfont *tf;
   color *gcol;
   color *fcol;
-  int no_init_string;
+  bool lacks_command_prefix;
   void tprint_start(troff_output_file *);
   void tprint_char(troff_output_file *, unsigned char);
   void tprint_end(troff_output_file *);
 public:
-  special_node(const macro &, int = 0);
-  special_node(const macro &, tfont *, color *, color *, statem *, int,
-	       int = 0);
+  device_extension_node(const macro & /* m */,
+			bool /* lacks_command_prefix */ = false);
+  device_extension_node(const macro & /* m */, tfont * /* tf */,
+			color * /* gcol */, color * /* fcol */,
+			statem * /* s */, int divlevel,
+			bool /* lacks_command_prefix */ = false);
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  hyphenation_type get_hyphenation_type();
   int ends_sentence();
   tfont *get_tfont();
+  void dump_properties();
 };
 
 class suppress_node : public node {
-  int is_on;
-  int emit_limits;	// must we issue the extent of the area written out?
+  int is_on; // three-valued Boolean :-|
+  bool emit_limits;	// must we issue extent of the area written out?
   symbol filename;
   char position;
   int  image_id;
@@ -541,13 +624,15 @@ public:
   suppress_node(symbol, char, int);
   suppress_node(int, int, symbol, char, int, statem *, int);
   suppress_node(int, int, symbol, char, int);
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
   hunits width();
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 private:
   void put(troff_output_file *, const char *);
 };
@@ -555,17 +640,19 @@ private:
 class tag_node : public node {
 public:
   string tag_string;
-  int delayed;
+  bool delayed;
   tag_node();
   tag_node(string, int);
   tag_node(string, statem *, int, int);
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
-  int ends_sentence();
+  bool causes_tprint();
+  bool is_tag();
+  int ends_sentence(); // three-valued Boolean :-|
+  void dump_properties();
 };
 
 struct hvpair {
@@ -583,58 +670,63 @@ class draw_node : public node {
   hvpair *point;
 public:
   draw_node(char, hvpair *, int, font_size, color *, color *);
-  draw_node(char, hvpair *, int, font_size, color *, color *, statem *, int);
+  draw_node(char, hvpair *, int, font_size, color *, color *, statem *,
+	    int);
   ~draw_node();
   hunits width();
   vunits vertical_width();
+  void asciify(macro *);
   node *copy();
   void tprint(troff_output_file *);
-  int same(node *);
+  bool is_same_as(node *);
   const char *type();
-  int force_tprint();
-  int is_tag();
+  bool causes_tprint();
+  bool is_tag();
+  void dump_properties();
 };
 
-class charinfo;
 node *make_node(charinfo *, environment *);
-int character_exists(charinfo *, environment *);
+bool character_exists(charinfo *, environment *);
 
 int same_node_list(node *, node *);
 node *reverse_node_list(node *);
 void delete_node_list(node *);
 node *copy_node_list(node *);
 
-int get_bold_fontno(int);
+hunits env_font_emboldening_offset(environment *, int);
 
 inline hyphen_list::hyphen_list(unsigned char code, hyphen_list *p)
-: hyphen(0), breakable(0), hyphenation_code(code), next(p)
+: is_hyphen(false), is_breakable(false), hyphenation_code(code), next(p)
 {
 }
 
 extern void read_desc();
-extern int mount_font(int, symbol, symbol = NULL_SYMBOL);
-extern int check_font(symbol, symbol);
-extern int check_style(symbol);
-extern void mount_style(int, symbol);
-extern int is_good_fontno(int);
+extern bool mount_font(int, symbol,
+		       symbol /* external_name */ = NULL_SYMBOL);
+extern bool is_font_name(symbol, symbol);
+extern bool is_abstract_style(symbol);
+extern bool mount_style(int, symbol);
+extern bool is_valid_font_mounting_position(int);
 extern int symbol_fontno(symbol);
 extern int next_available_font_position();
-extern void init_size_table(int *);
+extern void init_size_list(int *);
 extern int get_underline_fontno();
 
 class output_file {
   char make_g_plus_plus_shut_up;
 public:
   output_file();
+  bool is_dying;
   virtual ~output_file();
   virtual void trailer(vunits);
   virtual void flush() = 0;
   virtual void transparent_char(unsigned char) = 0;
   virtual void print_line(hunits x, vunits y, node *n,
-			  vunits before, vunits after, hunits width) = 0;
+			  vunits before, vunits after,
+			  hunits width) = 0;
   virtual void begin_page(int pageno, vunits page_length) = 0;
   virtual void copy_file(hunits x, vunits y, const char *filename) = 0;
-  virtual int is_printing() = 0;
+  virtual bool is_selected_for_printing() = 0;
   virtual void put_filename(const char *, int);
   virtual void on();
   virtual void off();
@@ -644,13 +736,11 @@ public:
   mtsm state;
 };
 
-#ifndef POPEN_MISSING
 extern char *pipe_command;
-#endif
 
 extern output_file *the_output;
 extern void init_output();
-int in_output_page_list(int);
+bool in_output_page_list(int);
 
 class font_family {
   int *map;
@@ -659,7 +749,7 @@ public:
   const symbol nm;
   font_family(symbol);
   ~font_family();
-  int make_definite(int);
+  int resolve(int);
   static void invalidate_fontno(int);
 };
 
@@ -667,3 +757,10 @@ font_family *lookup_family(symbol);
 symbol get_font_name(int, environment *);
 symbol get_style_name(int);
 extern search_path include_search_path;
+extern const int FONT_NOT_MOUNTED;
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
