@@ -1,8 +1,7 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
+/* Copyright 1989-2025 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -25,11 +24,15 @@ public:
   virtual void increment();
   virtual void decrement();
   virtual void set_increment(units);
+  virtual int get_increment() const;
   virtual void alter_format(char f, int w = 0);
   virtual const char *get_format();
   virtual void set_value(units);
+  virtual bool can_autoincrement() const;
 };
 
+// TODO: Use template to parameterize class in the pointed-to data type?
+
 class readonly_register : public reg {
   int *p;
 public:
@@ -37,6 +40,13 @@ public:
   const char *get_string();
 };
 
+class readonly_boolean_register : public reg {
+  bool *p;
+public:
+  readonly_boolean_register(bool *);
+  const char *get_string();
+};
+
 class general_reg : public reg {
   char format;
   int width;
@@ -48,6 +58,8 @@ public:
   void decrement();
   void alter_format(char f, int w = 0);
   void set_increment(units);
+  int get_increment() const;
+  bool can_autoincrement() const;
   const char *get_format();
   void add_value(units);
 
@@ -64,11 +76,17 @@ public:
 };
 
 extern object_dictionary register_dictionary;
-extern void set_number_reg(symbol nm, units n);
+extern void set_register(symbol nm, units n);
 extern void check_output_limits(int x, int y);
 extern void reset_output_registers();
 
-reg *lookup_number_reg(symbol);
+extern reg *look_up_register(symbol, bool suppress_creation = false);
 #if 0
-void inline_define_reg();
+void inline_define_register();
 #endif
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
