@@ -1,8 +1,7 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2018 Free Software Foundation, Inc.
+/* Copyright 1989-2025 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -24,14 +23,14 @@ class macro;
 class request_or_macro : public object {
 public:
   request_or_macro();
-  virtual void invoke(symbol, int) = 0;
+  virtual void invoke(symbol, bool) = 0;
   virtual macro *to_macro();
 };
 
 class request : public request_or_macro {
   REQUEST_FUNCP p;
 public:
-  void invoke(symbol, int);
+  void invoke(symbol, bool);
   request(REQUEST_FUNCP);
 };
 
@@ -46,15 +45,15 @@ class macro : public request_or_macro {
   const char *filename;		// where was it defined?
   int lineno;
   int len;
-  int empty_macro;
-  int is_a_diversion;
-  int is_a_string;		// if it contains no newline
+  bool is_empty_macro;
+  bool is_a_diversion;
+  bool is_a_string;		// if it contains no newline
 public:
   macro_header *p;
   macro();
   ~macro();
   macro(const macro &);
-  macro(int);
+  macro(bool /* is_div */);
   macro &operator=(const macro &);
   void append(unsigned char);
   void append(node *);
@@ -64,17 +63,19 @@ public:
   void set(unsigned char, int);
   unsigned char get(int);
   int length();
-  void invoke(symbol, int);
+  void invoke(symbol, bool);
   macro *to_macro();
   void print_size();
-  int empty();
-  int is_diversion();
-  int is_string();
+  bool is_empty();
+  bool is_diversion();
+  bool is_string();
   void clear_string_flag();
+  void dump();
+  void json_dump();
   friend class string_iterator;
   friend void chop_macro();
   friend void substring_request();
-  friend int operator==(const macro &, const macro &);
+  friend bool operator==(const macro &, const macro &);
 };
 
 extern void init_input_requests();
@@ -83,10 +84,17 @@ extern void init_div_requests();
 extern void init_node_requests();
 extern void init_reg_requests();
 extern void init_env_requests();
-extern void init_hyphen_requests();
+extern void init_hyphenation_pattern_requests();
 extern void init_request(const char *, REQUEST_FUNCP);
 
 class charinfo;
 class environment;
 
+// XXX: defined in input.cpp, but its only call site is in node.cpp
 node *charinfo_to_node_list(charinfo *, const environment *);
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
