@@ -1,8 +1,7 @@
-// -*- C++ -*-
-/* Copyright (C) 1994-2018 Free Software Foundation, Inc.
+/* Copyright (C) 1994-2025 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
-This file is part of groff.
+This file is part of groff, the GNU roff typesetting system.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -24,15 +23,25 @@ option to specify symbol sets to look in
 put filename in error messages (or fix lib)
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <assert.h>
+#include <errno.h>
+#include <math.h> // cos(), sin()
+#include <stdcountof.h>
+#include <stdio.h> // FILE, fclose(), fgets(), fopen(), fprintf(),
+		   // freopen(), printf(), sprintf()
+#include <stdlib.h> // atoi(), exit(), EXIT_FAILURE, EXIT_SUCCESS,
+		    // strtol()
+#include <string.h> // strchr(), strcmp(), strerror(), strlen(),
+		    // strrchr(), strtok()
+
+#include <getopt.h> // getopt_long()
+
 #include "lib.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <math.h>
-#include <errno.h>
-#include "assert.h"
 #include "posix.h"
 #include "errarg.h"
 #include "error.h"
@@ -43,7 +52,6 @@ put filename in error messages (or fix lib)
 extern "C" const char *Version_string;
 extern const char *hp_msl_to_unicode_code(const char *);
 
-#define SIZEOF(v) (sizeof(v)/sizeof(v[0]))
 #define equal(a, b) (strcmp(a, b) == 0)
 // only valid if is_uname(c) has returned true
 #define is_decomposed(c) strchr(c, '_')
@@ -179,7 +187,7 @@ struct name_list {
   char *name;
   name_list *next;
   name_list(const char *s, name_list *p) : name(strsave(s)), next(p) { }
-  ~name_list() { a_delete name; }
+  ~name_list() { delete[] name; }
 };
 
 struct symbol_set {
@@ -282,18 +290,20 @@ main(int argc, char **argv)
   program_name = argv[0];
 
   int opt;
-  int res = 1200;		// PCL unit of measure for cursor moves
-  int scalesize = 4;		// LaserJet 4 only allows 1/4 point increments
+  int res = 1200;	// PCL unit of measure for cursor moves
+  int scalesize = 4;	// LaserJet 4 only allows 1/4 point increments
   int unitwidth = 6350;
-  double ppi;			// points per inch
-  double upem;			// design units per em
+  double ppi;		// points per inch
+  double upem;		// design units per em
 
   static const struct option long_options[] = {
-    { "help", no_argument, 0, CHAR_MAX + 1 },
-    { "version", no_argument, 0, 'v' },
-    { NULL, 0, 0, 0 }
+    { "help", no_argument, 0 /* nullptr */, CHAR_MAX + 1 },
+    { "version", no_argument, 0 /* nullptr */, 'v' },
+    { 0 /* nullptr */, 0, 0 /* nullptr */, 0 }
   };
-  while ((opt = getopt_long(argc, argv, "adsqvi:", long_options, NULL)) != EOF) {
+  while ((opt = getopt_long(argc, argv, ":adsqvi:", long_options,
+			    0 /* nullptr */))
+	  != EOF)
     switch (opt) {
     case 'a':
       all_flag = YES;
@@ -313,19 +323,29 @@ main(int argc, char **argv)
       break;
     case 'v':
       printf("GNU hpftodit (groff) version %s\n", Version_string);
-      exit(0);
+      exit(EXIT_SUCCESS);
       break;
     case CHAR_MAX + 1: // --help
       usage(stdout);
-      exit(0);
+      exit(EXIT_SUCCESS);
       break;
     case '?':
+      if (optopt != 0)
+	error("unrecognized command-line option '%1'", char(optopt));
+      else
+	error("unrecognized command-line option '%1'",
+	      argv[(optind - 1)]);
       usage();
       break;
+    case ':':
+      error("command-line option '%1' requires an argument",
+           char(optopt));
+      usage(stderr);
+      exit(2);
+      break;
     default:
-      assert(0);
+      assert(0 == "unhandled case of command-line option");
     }
-  }
 
   if (debug_flag && argc - optind < 1)
     usage();
@@ -337,7 +357,7 @@ main(int argc, char **argv)
   if (debug_flag)
     dump_tags(f);
   if (!debug_flag && !read_map(argv[optind + 1], tfm_type))
-    exit(1);
+    exit(EXIT_FAILURE);
   else if (debug_flag && argc - optind > 1)
     read_map(argv[optind + 1], tfm_type);
   current_filename = NULL;
@@ -385,16 +405,25 @@ static void
 usage(FILE *stream)
 {
   fprintf(stream,
-	  "usage: %s [-s] [-a] [-q] [-i n] tfm_file map_file output_font\n"
-	  "       %s -d tfm_file [map_file]\n",
-	  program_name, program_name);
+"usage: %s [-aqs] [-i n] tfm-file map-file output-font\n"
+"usage: %s -d tfm-file [map-file]\n"
+"usage: %s {-v | --version}\n"
+"usage: %s --help\n",
+	  program_name, program_name, program_name, program_name);
+  if (stdout == stream)
+    fputs("\n"
+"Create a font description file for use with a Hewlett-Packard\n"
+"LaserJet 4-series (or newer) printer and the grolj4(1) output driver\n"
+"of groff(1), using data from an HP tagged font metric (TFM) file.\n"
+"See the hpftodit(1) manual page.\n",
+	  stream);
 }
 
 static void
 usage()
 {
   usage(stderr);
-  exit(1);
+  exit(2);
 }
 
 File::File(const char *s)
@@ -574,7 +603,7 @@ output_font_name(File &f)
     ;
   *(p + 1) = '\0';
   printf("# %s\n", font_name);
-  delete font_name;
+  delete[] font_name;
 }
 
 static void
@@ -800,7 +829,7 @@ output_ligatures()
     if (charcode < charcode_name_table_size
 	&& char_table[i].symbol_set != NO_SYMBOL_SET) {
       for (name_list *p = charcode_name_table[charcode]; p; p = p->next)
-	for (unsigned int j = 0; j < SIZEOF(ligature_chars); j++)
+	for (unsigned int j = 0; j < countof(ligature_chars); j++)
 	  if (strcmp(p->name, ligature_chars[j]) == 0) {
 	    ligature_mask |= 1 << j;
 	    break;
@@ -809,7 +838,7 @@ output_ligatures()
     }
   if (ligature_mask) {
     printf("ligatures");
-    for (i = 0; i < SIZEOF(ligature_names); i++)
+    for (i = 0; i < countof(ligature_names); i++)
       if (ligature_mask & (1 << i))
 	printf(" %s", ligature_names[i]);
     printf(" 0\n");
@@ -1256,17 +1285,19 @@ dump_symbols(int tfm_type)
 static char *
 show_symset(unsigned int symset)
 {
-   static char symset_str[8];
-
-   sprintf(symset_str, "%d%c", symset / 32, (symset & 31) + 64);
-   return symset_str;
+  // A 64-bit unsigned int produces up to 20 decimal digits.
+  assert(sizeof(unsigned int) <= 8);
+  static char symset_str[22]; // 20 digits + symset char + \0
+  sprintf(symset_str, "%u%c", symset / 32, (symset & 31) + 64);
+  return symset_str;
 }
 
 static char *
 hp_msl_to_ucode_name(int msl)
 {
-  char codestr[8];
-
+  // A 64-bit signed int produces up to 19 decimal digits plus a sign.
+  assert(sizeof(int) <= 8);
+  char codestr[21]; // 19 digits + possible sign + \0
   sprintf(codestr, "%d", msl);
   const char *ustr = hp_msl_to_unicode_code(codestr);
   if (ustr == NULL)
@@ -1285,15 +1316,17 @@ hp_msl_to_ucode_name(int msl)
       ustr = uname_decomposed + 1;
   }
   char *value = new char[strlen(ustr) + 1];
-  sprintf(value, equal(ustr, UNNAMED) ? ustr : "u%s", ustr);
+  sprintf(value, equal(ustr, UNNAMED) ? UNNAMED : "u%s", ustr);
   return value;
 }
 
 static char *
 unicode_to_ucode_name(int ucode)
 {
+  // A 64-bit signed int produces up to 16 hexadecimal digits.
+  assert(sizeof(int) <= 8);
   const char *ustr;
-  char codestr[8];
+  char codestr[sizeof "AAAABBBBCCCCDDDD"];
 
   // don't allow PUA code points as Unicode names
   if (ucode >= 0xE000 && ucode <= 0xF8FF)
@@ -1309,7 +1342,7 @@ unicode_to_ucode_name(int ucode)
       ustr = uname_decomposed + 1;
   }
   char *value = new char[strlen(ustr) + 1];
-  sprintf(value, equal(ustr, UNNAMED) ? ustr : "u%s", ustr);
+  sprintf(value, equal(ustr, UNNAMED) ? UNNAMED : "u%s", ustr);
   return value;
 }
 
@@ -1416,7 +1449,7 @@ read_map(const char *file, const int tfm_type)
       charcode_name_table = new name_list *[charcode_name_table_size];
       if (old_table) {
 	memcpy(charcode_name_table, old_table, old_size*sizeof(name_list *));
-	a_delete old_table;
+	delete[] old_table;
       }
       for (size_t i = old_size; i < charcode_name_table_size; i++)
 	charcode_name_table[i] = NULL;
@@ -1451,3 +1484,9 @@ xbasename(const char *s)
     }
   return b ? b + 1 : s;
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
