@@ -1,8 +1,8 @@
 #!/bin/sh
 #
-# Copyright (C) 2021 Free Software Foundation, Inc.
+# Copyright 2021-2025 G. Branden Robinson
 #
-# This file is part of groff.
+# This file is part of groff, the GNU roff typesetting system.
 #
 # groff is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -41,6 +41,10 @@ output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -men 2>&1)
 echo 'checking raw troff with -men' >&2
 echo "$output" | grep -Fqx '.hy=4' || wail
 
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mes 2>&1)
+echo 'checking raw troff with -mes' >&2
+echo "$output" | grep -Fqx '.hy=1' || wail
+
 output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mfr 2>&1)
 echo 'checking raw troff with -mfr' >&2
 echo "$output" | grep -Fqx '.hy=4' || wail
@@ -49,6 +53,14 @@ output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mit 2>&1)
 echo 'checking raw troff with -mit' >&2
 echo "$output" | grep -Fqx '.hy=1' || wail
 
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mpl 2>&1)
+echo 'checking raw troff with -mpl' >&2
+echo "$output" | grep -Fqx '.hy=1' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mru 2>&1)
+echo 'checking raw troff with -mru' >&2
+echo "$output" | grep -Fqx '.hy=1' || wail
+
 output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -msv 2>&1)
 echo 'checking raw troff with -msv' >&2
 echo "$output" | grep -Fqx '.hy=32' || wail
@@ -65,6 +77,10 @@ output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -men 2>&1)
 echo 'checking -me with -men' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
 
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -mes 2>&1)
+echo 'checking -me with -mes' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
 output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -mfr 2>&1)
 echo 'checking -me with -mfr' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
@@ -73,6 +89,14 @@ output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -mit 2>&1)
 echo 'checking -me with -mit' >&2
 echo "$output" | grep -Fqx '.hy=2' || wail
 
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -mpl 2>&1)
+echo 'checking -me with -mpl' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -mru 2>&1)
+echo 'checking -me with -mru' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
 output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -me -msv 2>&1)
 echo 'checking -me with -msv' >&2
 echo "$output" | grep -Fqx '.hy=34' || wail
@@ -89,6 +113,10 @@ output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -men 2>&1)
 echo 'checking -ms with -men' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
 
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -mes 2>&1)
+echo 'checking -ms with -mes' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
 output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -mfr 2>&1)
 echo 'checking -ms with -mfr' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
@@ -97,6 +125,14 @@ output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -mit 2>&1)
 echo 'checking -ms with -mit' >&2
 echo "$output" | grep -Fqx '.hy=2' || wail
 
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -mpl 2>&1)
+echo 'checking -ms with -mpl' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -mru 2>&1)
+echo 'checking -ms with -mru' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
 output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -ms -msv 2>&1)
 echo 'checking -ms with -msv' >&2
 echo "$output" | grep -Fqx '.hy=34' || wail
@@ -104,84 +140,171 @@ echo "$output" | grep -Fqx '.hy=34' || wail
 input='.TH foo 1 2022-01-06 "groff test suite"
 .tm .hy=\n[.hy]'
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -mcs 2>&1)
-echo 'checking -man with -rcR=0 -mcs' >&2
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mcs 2>&1)
+echo 'checking -man with -mcs' >&2
 echo "$output" | grep -Fqx '.hy=2' || wail
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -mde 2>&1)
-echo 'checking -man with -rcR=0 -mde' >&2
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mde 2>&1)
+echo 'checking -man with -mde' >&2
 echo "$output" | grep -Fqx '.hy=2' || wail
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -men 2>&1)
-echo 'checking -man with -rcR=0 -men' >&2
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -men 2>&1)
+echo 'checking -man with -men' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -mfr 2>&1)
-echo 'checking -man with -rcR=0 -mfr' >&2
-echo "$output" | grep -Fqx '.hy=6' || wail
-
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -mit 2>&1)
-echo 'checking -man with -rcR=0 -mit' >&2
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mes 2>&1)
+echo 'checking -man with -mes' >&2
 echo "$output" | grep -Fqx '.hy=2' || wail
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -msv 2>&1)
-echo 'checking -man with -rcR=0 -msv' >&2
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mfr 2>&1)
+echo 'checking -man with -mfr' >&2
+echo "$output" | grep -Fqx '.hy=6' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mit 2>&1)
+echo 'checking -man with -mit' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mpl 2>&1)
+echo 'checking -man with -mpl' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -mru 2>&1)
+echo 'checking -man with -mru' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -man -msv 2>&1)
+echo 'checking -man with -msv' >&2
 echo "$output" | grep -Fqx '.hy=34' || wail
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=1 -man -mcs 2>&1)
-echo 'checking -man with -rcR=1 -mcs' >&2
-echo "$output" | grep -Fqx '.hy=1' || wail
-
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=1 -man -mde 2>&1)
-echo 'checking -man with -rcR=1 -mde' >&2
-echo "$output" | grep -Fqx '.hy=1' || wail
-
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=1 -man -men 2>&1)
-echo 'checking -man with -rcR=1 -men' >&2
-echo "$output" | grep -Fqx '.hy=4' || wail
-
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=1 -man -mfr 2>&1)
-echo 'checking -man with -rcR=1 -mfr' >&2
-echo "$output" | grep -Fqx '.hy=4' || wail
-
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=1 -man -mit 2>&1)
-echo 'checking -man with -rcR=1 -mit' >&2
-echo "$output" | grep -Fqx '.hy=1' || wail
-
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=1 -man -msv 2>&1)
-echo 'checking -man with -rcR=1 -msv' >&2
-echo "$output" | grep -Fqx '.hy=32' || wail
-
 # Ensure that the 'trap bit' (hyphenation value 2, which has nothing to
 # do with any language) is preserved when switching locales back from a
 # CJK language, since those languages' modes unconditionally clear it.
 
-input='.TH foo 1 2022-04-09 "groff test suite"
+input='.
+.TH foo 1 2022-04-09 "groff test suite"
 .SH 名前
 foo \- APT 用選択制御ファイル
 .mso en.tmac
 .TH bar 1 2022-04-09 "groff test suite"
 .SH Name
 bar \- three subjects walk into this
-.tm .hy=\n[.hy]'
+.tm .hy=\n[.hy]
+.'
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -mja \
-  -men 2>&1)
-echo 'checking -man with -rcR=0 -mja -men' >&2
+output=$(printf "%s\n" "$input" | "$groff" -K utf8 -Tutf8 -P-cbou \
+  -man -mja -men 2>&1)
+echo 'checking -man with -mja -men' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
 
-input='.TH foo 1 2022-04-09 "groff test suite"
+input='.
+.TH foo 1 2022-04-09 "groff test suite"
 .SH 名称
 foo \- 解析 man 手册页的头部信息
 .mso en.tmac
 .TH bar 1 2022-04-09 "groff test suite"
 .SH Name
 bar \- three subjects walk into this
-.tm .hy=\n[.hy]'
+.tm .hy=\n[.hy]
+.'
 
-output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -rcR=0 -man -mzh \
-  -men 2>&1)
-echo 'checking -man with -rcR=0 -mzh -men' >&2
+output=$(printf "%s\n" "$input" | "$groff" -K utf8 -Tutf8 -P-cbou \
+  -man -mzh -men 2>&1)
+echo 'checking -man with -mzh -men' >&2
+echo "$output" | grep -Fqx '.hy=6' || wail
+
+input='.
+.Dd 2025-07-16
+.Dt foo 1
+.Os "groff test suite"
+.tm .hy=\n[.hy]
+.'
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mcs 2>&1)
+echo 'checking -mdoc with -mcs' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mde 2>&1)
+echo 'checking -mdoc with -mde' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -men 2>&1)
+echo 'checking -mdoc with -men' >&2
+echo "$output" | grep -Fqx '.hy=6' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mes 2>&1)
+echo 'checking -mdoc with -mes' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mfr 2>&1)
+echo 'checking -mdoc with -mfr' >&2
+echo "$output" | grep -Fqx '.hy=6' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mit 2>&1)
+echo 'checking -mdoc with -mit' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mpl 2>&1)
+echo 'checking -mdoc with -mpl' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -mru 2>&1)
+echo 'checking -mdoc with -mru' >&2
+echo "$output" | grep -Fqx '.hy=2' || wail
+
+output=$(printf "%s\n" "$input" | "$groff" -Tascii -P-cbou -mdoc -msv 2>&1)
+echo 'checking -mdoc with -msv' >&2
+echo "$output" | grep -Fqx '.hy=34' || wail
+
+# Ensure that the 'trap bit' (hyphenation value 2, which has nothing to
+# do with any language) is preserved when switching locales back from a
+# CJK language, since those languages' modes unconditionally clear it.
+
+input='.
+.Dd 2025-07-16
+.Dt foo 1
+.Os "groff test suite"
+.Sh 名前
+.Nm foo
+.Nd APT 用選択制御ファイル
+.mso en.tmac
+.Dd 2025-07-16
+.Dt bar 1
+.Os "groff test suite"
+.Sh Name
+.Nm bar
+.Nd three subjects walk into this
+.\".Sh Description
+.\"Stuff.
+.tm .hy=\n[.hy]
+.'
+
+output=$(printf "%s\n" "$input" | "$groff" -K utf8 -Tutf8 -P-cbou \
+  -mdoc -mja -men 2>&1)
+echo 'checking -mdoc with -mja -men' >&2
+echo "$output" | grep -Fqx '.hy=6' || wail
+
+input='.
+.Dd 2025-07-16
+.Dt foo 1
+.Os "groff test suite"
+.Sh 名称
+.Nm foo
+.Nd 解析 mdoc 手册页的头部信息
+.mso en.tmac
+.Dd 2025-07-16
+.Dt bar 1
+.Os "groff test suite"
+.Sh Name
+.Nm bar
+.Nd three subjects walk into this
+.\".Sh Description
+.\"Stuff.
+.tm .hy=\n[.hy]
+.'
+
+output=$(printf "%s\n" "$input" | "$groff" -K utf8 -Tutf8 -P-cbou \
+  -mdoc -mzh -men 2>&1)
+echo 'checking -mdoc with -mzh -men' >&2
 echo "$output" | grep -Fqx '.hy=6' || wail
 
 test -z "$fail"
