@@ -1,8 +1,8 @@
 #!/bin/sh
 #
-# Copyright (C) 2022 Free Software Foundation, Inc.
+# Copyright 2022 G. Branden Robinson
 #
-# This file is part of groff.
+# This file is part of groff, the GNU roff typesetting system.
 #
 # groff is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -16,7 +16,6 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
 
 groff="${abs_top_builddir:-.}/test-groff"
 # Give the output a name that won't collide with another test.
@@ -25,11 +24,14 @@ gnu_pdf="${abs_top_builddir:-.}/doc/gnu-no-choke-on-pdfinfo.pdf"
 # Regression-test Savannah #58206.
 
 # We need gs(1) and pdfpic.tmac needs pdfinfo(1).
+# TODO: Generate this test script from an '.in' file and search for the
+# `GHOSTSCRIPT` command name detected by our "configure" script.
+# See "contrib/hdtbl/examples/test-hdtbl.sh.in".
 for cmd in gs pdfinfo
 do
     if ! command -v $cmd >/dev/null
     then
-        echo "cannot locate '$cmd' command" >&2
+        echo "$0: cannot locate '$cmd' command; skipping" >&2
         exit 77 # skip
     fi
 done
@@ -49,11 +51,16 @@ do
 done
 
 # If we can't find it, we can't test.
-test -z "$artifact_dir" && exit 77 # skip
+if [ -z "$artifact_dir" ]
+then
+    echo "$0: cannot locate test artifact directory; skipping" >&2
+    exit 77 # skip
+fi
 
 if [ -e "$gnu_pdf" ]
 then
-    echo "temporary output file '$gnu_pdf' already exists" >&2
+    echo "$0: temporary output file '$gnu_pdf' already exists;" \
+        "skipping" >&2
     exit 77 # skip
 fi
 
@@ -80,4 +87,4 @@ test -z "$fail" \
 rm -f "$gnu_pdf"
 test -z "$fail"
 
-# vim:set ai et sw=4 ts=4 tw=72:
+# vim:set autoindent expandtab shiftwidth=4 tabstop=4 textwidth=72:
